<?php

namespace Drupal\s3_db_export;

use Aws\S3\S3Client;
use Drupal\Core\Database\Connection;
use Drupal\Core\File\FileSystem;
use Drupal\Core\State\State;
use Ifsnop\Mysqldump\Mysqldump;

/**
 * Define SosManager class of Bi_sos custom module.
 *
 * @todo Rename to DatabaseExportManagerS3
 */
class DbexportManager implements DbexportManagerInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * The filesystem service.
   *
   * @var \Symfony\Component\Filesystem\Filesystem
   */
  protected $fileSystem;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\State
   */
  protected $state;

  /**
   * Constructor method.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Symfony\Component\Filesystem\Filesystem $fileSystem
   *   The filesystem service.
   * @param \Drupal\Core\State\State $state
   *   The state service.
   */
  public function __construct(Connection $connection, FileSystem $fileSystem, State $state) {
    $this->connection = $connection;
    $this->fileSystem = $fileSystem;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public function variableGet($fist_param, $sec_param = "") {
    // @todo Use config instead of state.
    $r = $this->state->get($fist_param);
    return ($r == NULL) ? $sec_param : $r;
  }

  /**
   * Database dump.
   *
   * @todo Rename this function to drupal_db_dump to make it clearer.
   */
  public function dump() {
    $connection = $this->connection->getConnectionOptions();

    // @todo We should be using the private directory AND deleting the file
    //   after it's used, I didn't see the file being deleted.
    $directory = $this->fileSystem->realpath("public://temp/");

    $dump = new Mysqldump('mysql:host=' . $connection["host"] . ';dbname=' . $connection["database"], $connection["username"], $connection["password"]);
    $filename = 'backup_' . time() . rand(0, 999) . '.sql';
    $dump->start($directory . '/' . $filename);
    // \Drupal::logger('s3_db_export')->info('Database Dump Started');
    return $filename;
  }

  /**
   * Database export.
   *
   * @todo Rename to DatabaseExportS3
   */
  public function dbExport() {
    $dump = $this->dump();
    $this->awss3($dump);
    die;

  }

  /**
   * Move database in to AWS server.
   *
   * @todo Rename function to AWS_S3_Upload
   */
  public function awss3($filename) {
    $directory = $this->fileSystem->realpath("public://temp/");
    // @todo Use config instead of state.
    $s3 = new S3Client([
      'region' => $this->variableGet("aws_region", ""),
      'version' => $this->variableGet("aws_version", ""),
      'credentials' => [
        'key' => $this->variableGet("aws_key", ""),
        'secret' => $this->variableGet("aws_secret", ""),
      ],
    ]);

    // @todo Use config instead of state.
    $aws_bucket_folder = $this->variableGet("aws_bucket_folder", "");
    $file_name = $aws_bucket_folder . '/' . $filename;
    $temp_file_location = $directory . '/' . $filename;

    $result = $s3->putObject([
      // @todo Use config instead of state.
      'Bucket' => $this->variableGet("aws_bucket", ""),
      'Key' => $file_name,
      'SourceFile' => $temp_file_location,
    ]);
    if ($result) {
      // \Drupal::logger('s3_db_export')->info('Uploaded database in
      // aws via cron');
    }
    else {
      // \Drupal::logger('s3_db_export')->error('Something went wrong');
    }

    // \Drupal::logger('s3_db_export')->info($result);
  }

}
